@@ -24,6 +24,8 @@ extern "C" {
 #include <glob.h>
 #include <unistd.h>
 #include <sys/utsname.h>
+#include <sys/times.h>
+#include <sys/time.h>
 #include <time.h>
 }
 
@@ -36,7 +38,7 @@ version ()
   clog
     << "SystemTap translator/driver "
     << "(version " << VERSION << " built " << DATE << ")" << endl
-    << "Copyright (C) 2005 Red Hat, Inc. and others" << endl
+    << "Copyright (C) 2005-2006 Red Hat, Inc. and others" << endl
     << "This is free software; see the source for copying conditions."
     << endl;
 }
@@ -56,11 +58,11 @@ usage (systemtap_session& s)
     << endl
     << "Options:" << endl
     << "   --         no more options after this" << endl
-    << "   -v         verbose" << (s.verbose ? " [set]" : "") << endl
+    << "   -v         increase verbosity [" << s.verbose << "]" << endl
     << "   -h         show help" << endl
     << "   -V         show version" << endl
     << "   -k         keep temporary directory" << endl
-    // << "   -t         test mode" << (s.test_mode ? " [set]" : "") << endl
+    << "   -u         unoptimized translation" << (s.unoptimized ? " [set]" : "") << endl
     << "   -g         guru mode" << (s.guru_mode ? " [set]" : "") << endl
     << "   -b         bulk (relayfs) mode" << (s.bulk_mode ? " [set]" : "") << endl
     << "   -s NUM     buffer size in megabytes, instead of "
@@ -119,10 +121,10 @@ main (int argc, char * const argv [])
   (void) uname (& buf);
   s.kernel_release = string (buf.release);
   s.architecture = string (buf.machine);
-  s.verbose = false;
-  s.test_mode = false;
+  s.verbose = 0;
   s.guru_mode = false;
   s.bulk_mode = false;
+  s.unoptimized = false;
   s.buffer_size = 0;
   s.last_pass = 5;
   s.module_name = "stap_" + stringify(getpid());
@@ -145,7 +147,7 @@ main (int argc, char * const argv [])
 
   while (true)
     {
-      int grc = getopt (argc, argv, "hVvp:I:e:o:tR:r:m:kgc:x:D:bs:");
+      int grc = getopt (argc, argv, "hVvp:I:e:o:R:r:m:kgc:x:D:bs:u");
       if (grc < 0)
         break;
       switch (grc)
@@ -155,7 +157,7 @@ main (int argc, char * const argv [])
           exit (0);
 
         case 'v':
-	  s.verbose = true;
+	  s.verbose ++;
 	  break;
 
         case 'p':
@@ -182,10 +184,6 @@ main (int argc, char * const argv [])
           s.output_file = string (optarg);
           break;
 
-        case 't':
-          s.test_mode = true;
-          break;
-
         case 'R':
           s.runtime_path = string (optarg);
           break;
@@ -210,6 +208,10 @@ main (int argc, char * const argv [])
           s.bulk_mode = true;
           break;
 
+	case 'u':
+	  s.unoptimized = true;
+	  break;
+
         case 's':
           s.buffer_size = atoi (optarg);
           if (s.buffer_size < 1 || s.buffer_size > 64)
@@ -283,10 +285,14 @@ main (int argc, char * const argv [])
     else
       s.tmpdir = tmpdir;
 
-    if (s.verbose)
+    if (s.verbose>1)
       clog << "Created temporary directory \"" << s.tmpdir << "\"" << endl;
   }
 
+  struct tms tms_before;
+  times (& tms_before);
+  struct timeval tv_before;
+  gettimeofday (&tv_before, NULL);
 
   // PASS 1a: PARSING USER SCRIPT
   // XXX: pass args vector, so parser (or lexer?) can substitute
@@ -342,7 +348,7 @@ main (int argc, char * const argv [])
             rc ++;
           // GLOB_NOMATCH is acceptable
 
-          if (s.verbose)
+          if (s.verbose>1)
             clog << "Searched '" << dir << "', "
                  << "match count " << globbuf.gl_pathc << endl;
 
@@ -373,10 +379,29 @@ main (int argc, char * const argv [])
           }
     }
 
+  struct tms tms_after;
+  times (& tms_after);
+  unsigned _sc_clk_tck = sysconf (_SC_CLK_TCK);
+  struct timeval tv_after;
+  gettimeofday (&tv_after, NULL);
+
+#define TIMESPRINT \
+           (tms_after.tms_cutime + tms_after.tms_utime \
+            - tms_before.tms_cutime - tms_before.tms_utime) * 1000 / (_sc_clk_tck) << "usr/" \
+        << (tms_after.tms_cstime + tms_after.tms_stime \
+            - tms_before.tms_cstime - tms_before.tms_stime) * 1000 / (_sc_clk_tck) << "sys/" \
+        << ((tv_after.tv_sec - tv_before.tv_sec) * 1000 + \
+            ((long)tv_after.tv_usec - (long)tv_before.tv_usec) / 1000) << "real ms."
+
   // syntax errors, if any, are already printed
-  if (s.verbose) clog << "Pass 1: parsed user script and "
-                      << s.library_files.size()
-                      << " library script(s)." << endl;
+  if (s.verbose)
+    {
+      clog << "Pass 1: parsed user script and "
+           << s.library_files.size()
+           << " library script(s) in "
+           << TIMESPRINT
+           << endl;
+    }
 
   if (rc)
     cerr << "Pass 1: parse failed.  "
@@ -385,6 +410,9 @@ main (int argc, char * const argv [])
 
   if (rc || s.last_pass == 1) goto cleanup;
 
+  times (& tms_before);
+  gettimeofday (&tv_before, NULL);
+
   // PASS 2: ELABORATION
   rc = semantic_pass (s);
 
@@ -415,6 +443,11 @@ main (int argc, char * const argv [])
               v->printsig (cout);
               cout << endl;
             }
+          if (s.verbose)
+            {
+              f->body->print (cout);
+              cout << endl;
+            }
 	}
 
       if (s.probes.size() > 0)
@@ -433,13 +466,23 @@ main (int argc, char * const argv [])
               v->printsig (cout);
               cout << endl;
             }
+          if (s.verbose)
+            {
+              p->body->print (cout);
+              cout << endl;
+            }
 	}
     }
 
-  if (s.verbose) clog << "Pass 2: analyzed user script.  "
+  times (& tms_after);
+  gettimeofday (&tv_after, NULL);
+
+  if (s.verbose) clog << "Pass 2: analyzed script: "
                       << s.probes.size() << " probe(s), "
                       << s.functions.size() << " function(s), "
-                      << s.globals.size() << " global(s)." << endl;
+                      << s.globals.size() << " global(s) in "
+                      << TIMESPRINT
+                      << endl;
 
   if (rc)
     cerr << "Pass 2: analysis failed.  "
@@ -449,6 +492,10 @@ main (int argc, char * const argv [])
   if (rc || s.last_pass == 2) goto cleanup;
 
   // PASS 3: TRANSLATION
+
+  times (& tms_before);
+  gettimeofday (&tv_before, NULL);
+
   s.translated_source = string(s.tmpdir) + "/" + s.module_name + ".c";
   rc = translate_pass (s);
 
@@ -458,9 +505,14 @@ main (int argc, char * const argv [])
       cout << i.rdbuf();
     }
 
+  times (& tms_after);
+  gettimeofday (&tv_after, NULL);
+
   if (s.verbose) clog << "Pass 3: translated to C into \""
                       << s.translated_source
-                      << "\"" << endl;
+                      << "\" in "
+                      << TIMESPRINT
+                      << endl;
 
   if (rc)
     cerr << "Pass 3: translation failed.  "
@@ -470,7 +522,17 @@ main (int argc, char * const argv [])
   if (rc || s.last_pass == 3) goto cleanup;
 
   // PASS 4: COMPILATION
+  times (& tms_before);
+  gettimeofday (&tv_before, NULL);
   rc = compile_pass (s);
+  times (& tms_after);
+  gettimeofday (&tv_after, NULL);
+
+  if (s.verbose) clog << "Pass 4: compiled C into \""
+                      << s.module_name << ".ko"
+                      << "\" in "
+                      << TIMESPRINT
+                      << endl;
 
   if (rc)
     cerr << "Pass 4: compilation failed.  "
@@ -481,7 +543,14 @@ main (int argc, char * const argv [])
   if (rc || s.last_pass == 4) goto cleanup;
 
   // PASS 5: RUN
+  times (& tms_before);
+  gettimeofday (&tv_before, NULL);
   rc = run_pass (s);
+  times (& tms_after);
+  gettimeofday (&tv_after, NULL);
+  if (s.verbose) clog << "Pass 5: run completed in "
+                      << TIMESPRINT
+                      << endl;
 
   if (rc)
     cerr << "Pass 5: run failed.  "
@@ -502,9 +571,9 @@ main (int argc, char * const argv [])
         {
           string cleanupcmd = "rm -rf ";
           cleanupcmd += s.tmpdir;
-          if (s.verbose) clog << "Running " << cleanupcmd << endl;
+          if (s.verbose>1) clog << "Running " << cleanupcmd << endl;
 	  int status = system (cleanupcmd.c_str());
-	  if (status != 0 && s.verbose)
+	  if (status != 0 && s.verbose>1)
 	    clog << "Cleanup command failed, status: " << status << endl;
         }
     }
