@@ -19,6 +19,8 @@ static struct rchan *_stp_chan;
 static struct dentry *_stp_dir;
 #endif
 
+static DECLARE_MUTEX(_stp_start_mutex);
+
 static int _stp_dpid;
 static int _stp_pid;
 
@@ -27,14 +29,15 @@ MODULE_PARM_DESC(_stp_pid, "daemon pid");
 
 int _stp_target = 0;
 int _stp_exit_called = 0;
+int _stp_exit_flag = 0;
 
 /* forward declarations */
 void probe_exit(void);
 int probe_start(void);
 void _stp_exit(void);
 void _stp_handle_start (struct transport_start *st);
-static void _stp_handle_exit (void *data);
-static DECLARE_WORK(stp_exit, _stp_handle_exit, NULL);
+static void _stp_work_queue (void *data);
+static DECLARE_WORK(stp_exit, _stp_work_queue, NULL);
 int _stp_transport_open(struct transport_info *info);
 
 #include "procfs.c"
@@ -75,9 +78,15 @@ int _stp_transport_write (void *data, int len)
 static void _stp_handle_buf_info(int *cpuptr)
 {
 	struct buf_info out;
+
 	out.cpu = *cpuptr;
+#ifdef RELAYFS_VERSION_GE_4
+	out.produced = _stp_chan->buf[*cpuptr]->subbufs_produced;
+	out.consumed = _stp_chan->buf[*cpuptr]->subbufs_consumed;
+#else
 	out.produced = atomic_read(&_stp_chan->buf[*cpuptr]->subbufs_produced);
 	out.consumed = atomic_read(&_stp_chan->buf[*cpuptr]->subbufs_consumed);
+#endif /* RELAYFS_VERSION_GE_4 */
 
 	_stp_transport_send(STP_BUF_INFO, &out, sizeof(out));
 }
@@ -89,7 +98,11 @@ static void _stp_handle_buf_info(int *cpuptr)
 void _stp_handle_start (struct transport_start *st)
 {
 	kbug ("stp_handle_start pid=%d\n", st->pid);
+
+	down (&_stp_start_mutex);
 	st->pid = probe_start();
+	up (&_stp_start_mutex);
+
 	if (st->pid < 0) 
 		_stp_exit_called = 1;
 	_stp_transport_send(STP_START, st, sizeof(*st));
@@ -130,11 +143,29 @@ static void _stp_cleanup_and_exit (int closing)
 }
 
 /*
- *	_stp_handle_exit - handle STP_EXIT
+ *	_stp_work_queue - periodically check for IO or exit
  */
-static void _stp_handle_exit (void *data)
+static void _stp_work_queue (void *data)
 {
-	_stp_cleanup_and_exit(0);
+	int do_io = 0;
+
+	spin_lock(&_stp_ready_lock);
+	if (!list_empty(&_stp_ready_q))
+		do_io = 1;
+	spin_unlock(&_stp_ready_lock);
+
+	if (do_io)
+		wake_up_interruptible(&_stp_proc_wq);
+
+	if (_stp_exit_flag) {
+		cancel_delayed_work(&stp_exit);
+		down (&_stp_start_mutex);
+		_stp_cleanup_and_exit(0);
+		up (&_stp_start_mutex);
+		wake_up_interruptible(&_stp_proc_wq);
+	} else
+		schedule_delayed_work(&stp_exit, STP_WORK_TIMER);
+
 }
 
 /**
@@ -175,7 +206,7 @@ void _stp_transport_close()
 
 int _stp_transport_open(struct transport_info *info)
 {
-	kbug ("stp_transport_open: %d byte buffer. target=%d\n", info->buf_size, info->target);
+	kbug ("stp_transport_open: %d Mb buffer. target=%d\n", info->buf_size, info->target);
 
 	info->transport_mode = _stp_transport_mode;
 	kbug("transport_mode=%d\n", info->transport_mode);
@@ -196,6 +227,8 @@ int _stp_transport_open(struct transport_info *info)
 			_stp_unregister_procfs();
 			return -ENOMEM;
 		}
+		kbug ("stp_transport_open: %u Mb buffers, subbuf_size=%u, n_subbufs=%u\n",
+		      info->buf_size, subbuf_size, n_subbufs);
 	} else 
 #endif
 	{
@@ -216,6 +249,7 @@ int _stp_transport_init(void)
 	kbug("transport_init from %ld %ld\n", (long)_stp_pid, (long)current->pid);
 
 	_stp_register_procfs();
+	schedule_delayed_work(&stp_exit, STP_WORK_TIMER);
 	return 0;
 }
 
